#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess

# Local import
from utils.utils_files import *

"""
Class: Spatial completeness checks
---
- Checks for data gaps or missing values
"""
class spatial_completeness_test:

    def __init__(self, logdir, path, pattern, typef):
        self.testid = 'test02-spatial-completeness'
        self.logdir = logdir           
        self.type = typef
        self.path = path
        self.files = dir_or_file(path, pattern)

    def missing_values_netcdf(self, ncitem, e):
        var = ncitem.variables[e]
        variable = var[:]
        missing = variable.missing_value
        if var.mask == False:
            # If the variable does not have a mask
            datamiss = len(variable[variable==missing])
        else:
            # If the variable does have a mask
            data_masked = variable.mask
            data = variable.data
            real_data = data[~data_masked]
            datamiss = len(real_data[real_data == missing])

        mk = datamiss > 0

    def netcdf_test(self, args):
        import netCDF4
        import numpy as np
        
        # Progress init
        i = 0
        n = len(args)
        print()
        output = list()
        
        for ncfile in args:

            # Progress update
            i+=1
            print_progress(i, n)  

            ncitem = netCDF4.Dataset(f)
            test_var = [item for item in ncitem.variables.keys() \
                if item not in ncitem.dimensions.keys()]
            if test_var != []:
                for e in test_var:
                    datamiss,mk = self.missing_values_netcdf(ncitem,e)
                    if mk == True:
                        string = ncfile.split('/')[-1] + ' variable '
                        string += e + ' has missing data ' + datamiss
                        output.append('\n'.join([ncfile.split('/')[-1],string,e]))

        if len(output) == 0:
            stringA = '\nNO GAPS FOUND!'
            logging.info(stringA)

        else:
            logging.error('n' + '\n\n'.join(output))

    def missing_values_grib(self, msgs):
        # You should check how to see if a message has a mask
        output = list()
        for i, msg in enumerate(msgs):
            vals = msg['values']
            valmiss = msg['missingValue']
            numofmisscalc = len(vals[vals==valmiss])
            numofmiss = msg['numberOfMissing']

            if numofmisscalc == numofmiss and numofmiss > 0:
                output.append('Message number %s has %s num of missing values' %(i+1,numofmiss))
            
        if len(output):
            return '\n' + '\n'.join(output), 'ER'

        else:
            return '\nNo missing values', 'OK'

    def grib_test(self, args):
        import utils.grib_content as gcont

        # Progress init
        i = 0
        n = len(args)
        print()        
        oute = list()

        for item in args:
            
            # Progress update
            i+=1
            print_progress(i, n)  

            gfile = gcont.grib_content(item)
            msgs = gfile.return_all_msg()

            string, wrn = self.missing_values_grib(msgs)
            print(wrn)
            if wrn == 'ER':
                stringA = '\n' + '-'*80
                stringA += '\nFILE %s has missing values. More precisely\n\n' %(item)
                stringA += string

            try:
                oute.append(stringA)
            except:
                stringA = oute

        if not len(oute):
            logging.info('\nNO GAPS FOUND! HURRAY!')

        else:
            stringB = 'n' + '\n\n'.join(oute)
            print("Some errors were found. Please check the log file")            
            logging.error(stringA)

    def main(self):
        if self.type != None:                

            # Initiate logging
            initiate_logging(self.logdir, self.path, self.testid)
            
            if self.type == 'netcdf' or self.type == 'CF':
                ncfiles = [item for item in self.files if item.endswith('.nc')]
                if ncfiles != []:                    
                    self.netcdf_test(ncfiles)
                else:
                    raise Exception('\n\tNo NetCDF files in the given input\n')

            elif self.type == 'grib':
                gribfiles = [item for item in self.files if item.endswith('.grib')]
                if gribfiles != []:                    
                    self.grib_test(gribfiles)
                else:
                    raise Exception('\n\tNo GRIB files in the given input\n')
            else:
                raise Exception("Type argument not supported for this test")

        else:
            raise Exception('\n\tStandard type has not been given by the user\n')